package com.zanthan.sequence.swing.editor;

import java.awt.Container;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

public abstract class ChooseParserDialogBase extends JDialog {

    protected ResourceBundle resources;
    private JScrollPane scrollParserList;
    private JList parserList;
    private JScrollPane scrollParserDescription;
    private JTextArea parserDescription;
    private JPanel buttonPanel;
    private JPanel buttonGroup;
    private JButton okButton;
    private JButton cancelButton;


    public ChooseParserDialogBase() throws HeadlessException {
    }

    public ChooseParserDialogBase(Dialog owner) throws HeadlessException {
        super(owner);
    }

    public ChooseParserDialogBase(Dialog owner, String title, boolean modal, GraphicsConfiguration gc) throws HeadlessException {
        super(owner, title, modal, gc);
    }

    public ChooseParserDialogBase(Frame owner) throws HeadlessException {
        super(owner);
    }

    public ChooseParserDialogBase(Frame owner, String title, boolean modal, GraphicsConfiguration gc) {
        super(owner, title, modal, gc);
    }
  

    protected void init(Locale locale) {
        resources = new InnerResourceBundle(ResourceBundle.getBundle("com.zanthan.sequence.swing.editor.ChooseParserDialogBase", locale), getResources(locale));
        this.setTitle(resources.getString("this.title"));
        this.setModal(true);
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                closeDialog(event);
           }
        });
        
        addChildren(getContentPane());
    }
 
    /**
     * Implement and return any resources you want to have accessible
     * from the ResourceBundle in the resources field. You can also 
     * override resources generated from the NetBeans form by providing 
     * a different value for the key in the resources returned by this 
     * method. Just return null if you're happy with the default resources.
     * @param locale the local the resources are for
     * @return a resource bundle or null
     */
    protected abstract ResourceBundle getResources(Locale locale);


    protected abstract void closeDialog(WindowEvent event);

    protected void addChildren(Container container) {
        GridBagConstraints constraints;
        GridBagLayout layout = new GridBagLayout();
        container.setLayout(layout);
             
        constraints = new GridBagConstraints(0, -1, 1, 1, 1.0, 0.0, 10, 1, new Insets(0, 0, 0, 0), 0, 0);
        layout.setConstraints(getScrollParserList(), constraints);
        container.add(getScrollParserList());
     
        constraints = new GridBagConstraints(0, -1, 1, 1, 1.0, 1.0, 10, 1, new Insets(0, 0, 0, 0), 0, 0);
        layout.setConstraints(getScrollParserDescription(), constraints);
        container.add(getScrollParserDescription());
     
        constraints = new GridBagConstraints(0, -1, 1, 1, 1.0, 0.0, 10, 2, new Insets(0, 0, 0, 0), 0, 0);
        layout.setConstraints(getButtonPanel(), constraints);
        container.add(getButtonPanel());

    }


    protected JScrollPane getScrollParserList() {
        if (scrollParserList == null) {
            scrollParserList = newScrollParserList();
            setScrollParserListProperties(scrollParserList);
            addScrollParserListChildren(scrollParserList);
            addScrollParserListEventHandlers(scrollParserList);
        }
        return scrollParserList;
    }

     protected JScrollPane newScrollParserList() {
        JScrollPane container = new JScrollPane();
        
        return container;
   }

    protected void setScrollParserListProperties(JScrollPane scrollParserList) {
        scrollParserList.setVerticalScrollBarPolicy(22);
        
    }

    protected void addScrollParserListEventHandlers(JScrollPane scrollParserList) {
        
    }

    protected void addScrollParserListChildren(JScrollPane container) {
        container.setViewportView(getParserList());

    }

    protected JScrollPane getScrollParserDescription() {
        if (scrollParserDescription == null) {
            scrollParserDescription = newScrollParserDescription();
            setScrollParserDescriptionProperties(scrollParserDescription);
            addScrollParserDescriptionChildren(scrollParserDescription);
            addScrollParserDescriptionEventHandlers(scrollParserDescription);
        }
        return scrollParserDescription;
    }

     protected JScrollPane newScrollParserDescription() {
        JScrollPane container = new JScrollPane();
        
        return container;
   }

    protected void setScrollParserDescriptionProperties(JScrollPane scrollParserDescription) {
        scrollParserDescription.setVerticalScrollBarPolicy(22);
        
    }

    protected void addScrollParserDescriptionEventHandlers(JScrollPane scrollParserDescription) {
        
    }

    protected void addScrollParserDescriptionChildren(JScrollPane container) {
        container.setViewportView(getParserDescription());

    }

    protected JPanel getButtonPanel() {
        if (buttonPanel == null) {
            buttonPanel = newButtonPanel();
            setButtonPanelProperties(buttonPanel);
            addButtonPanelChildren(buttonPanel);
            addButtonPanelEventHandlers(buttonPanel);
        }
        return buttonPanel;
    }

     protected JPanel newButtonPanel() {
        JPanel container = new JPanel();
        
        return container;
   }

    protected void setButtonPanelProperties(JPanel buttonPanel) {
        
    }

    protected void addButtonPanelEventHandlers(JPanel buttonPanel) {
        
    }

    protected void addButtonPanelChildren(JPanel container) {  
        FlowLayout layout = new FlowLayout();
        layout.setAlignment(2);
        
        container.setLayout(layout);
        
        container.add(getButtonGroup());

    }

    protected JPanel getButtonGroup() {
        if (buttonGroup == null) {
            buttonGroup = newButtonGroup();
            setButtonGroupProperties(buttonGroup);
            addButtonGroupChildren(buttonGroup);
            addButtonGroupEventHandlers(buttonGroup);
        }
        return buttonGroup;
    }

     protected JPanel newButtonGroup() {
        JPanel container = new JPanel();
        
        return container;
   }

    protected void setButtonGroupProperties(JPanel buttonGroup) {
        
    }

    protected void addButtonGroupEventHandlers(JPanel buttonGroup) {
        
    }

    protected void addButtonGroupChildren(JPanel container) {  
        GridLayout layout = new GridLayout();
        layout.setColumns(0);
        layout.setRows(1);
        
        container.setLayout(layout);
        
        container.add(getOkButton());

        container.add(getCancelButton());

    }


    protected JList getParserList() {
        if (parserList == null) {
            parserList = newParserList();
            setParserListProperties(parserList);
            addParserListEventHandlers(parserList);
        }
        return parserList;
    }

     protected JList newParserList() {
        JList component = new JList(new DefaultListModel());
        return component;
   }

    protected void setParserListProperties(JList parserList) {
        parserList.setSelectionMode(0);
        
    }

    protected void addParserListEventHandlers(JList parserList) {
        
    }

    protected JTextArea getParserDescription() {
        if (parserDescription == null) {
            parserDescription = newParserDescription();
            setParserDescriptionProperties(parserDescription);
            addParserDescriptionEventHandlers(parserDescription);
        }
        return parserDescription;
    }

     protected JTextArea newParserDescription() {
        JTextArea component = new JTextArea();
        return component;
   }

    protected void setParserDescriptionProperties(JTextArea parserDescription) {
        parserDescription.setLineWrap(true);
        parserDescription.setRows(5);
        parserDescription.setWrapStyleWord(true);
        
    }

    protected void addParserDescriptionEventHandlers(JTextArea parserDescription) {
        
    }

    protected JButton getOkButton() {
        if (okButton == null) {
            okButton = newOkButton();
            setOkButtonProperties(okButton);
            addOkButtonEventHandlers(okButton);
        }
        return okButton;
    }

     protected JButton newOkButton() {
        JButton component = new JButton();
        return component;
   }

    protected void setOkButtonProperties(JButton okButton) {
        okButton.setText(resources.getString("okButton.text"));
        
    }

    protected void addOkButtonEventHandlers(JButton okButton) {
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                okButtonActionPerformed(event);
           }
        });
        
    }

    protected abstract void okButtonActionPerformed(ActionEvent event);

    protected JButton getCancelButton() {
        if (cancelButton == null) {
            cancelButton = newCancelButton();
            setCancelButtonProperties(cancelButton);
            addCancelButtonEventHandlers(cancelButton);
        }
        return cancelButton;
    }

     protected JButton newCancelButton() {
        JButton component = new JButton();
        return component;
   }

    protected void setCancelButtonProperties(JButton cancelButton) {
        cancelButton.setText(resources.getString("cancelButton.text"));
        
    }

    protected void addCancelButtonEventHandlers(JButton cancelButton) {
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                cancelButtonActionPerformed(event);
           }
        });
        
    }

    protected abstract void cancelButtonActionPerformed(ActionEvent event);



    private class InnerResourceBundle extends ResourceBundle {

        private ResourceBundle child;

        public InnerResourceBundle(ResourceBundle parent, ResourceBundle child) {
            this.child = child;
            setParent(parent);
        }

        /**
         * Returns an enumeration of the keys.
         *
         */
        public Enumeration getKeys() {
            final Set keys = new HashSet();
            Enumeration parentKeys = parent.getKeys();
            while (parentKeys.hasMoreElements()) {
                Object o = (Object) parentKeys.nextElement();
                keys.add(o);
            }
            if (child != null) {
                Enumeration childKeys = child.getKeys();
                while (childKeys.hasMoreElements()) {
                    Object o = (Object) childKeys.nextElement();
                    keys.add(o);
                }
            }
            return new IteratorWrapper(keys.iterator());
        }

        /**
         * Gets an object for the given key from this resource bundle.
         * Returns null if this resource bundle does not contain an
         * object for the given key.
         *
         * @param key the key for the desired object
         * @exception NullPointerException if <code>key</code> is <code>null</code>
         * @return the object for the given key, or null
         */
        protected Object handleGetObject(String key) {
            if (child != null) {
                try {
                    return child.getObject(key);
                } catch (MissingResourceException e) {
                    return null;
                }
            } else {
                return null;
            }
        }

        private class IteratorWrapper implements Enumeration {
            private Iterator iterator;

            public IteratorWrapper(Iterator iterator) {

                this.iterator = iterator;
            }

            /**
             * Tests if this enumeration contains more elements.
             *
             * @return  <code>true</code> if and only if this enumeration object
             *           contains at least one more element to provide;
             *          <code>false</code> otherwise.
             */
            public boolean hasMoreElements() {
                return iterator.hasNext();
            }

            /**
             * Returns the next element of this enumeration if this enumeration
             * object has at least one more element to provide.
             *
             * @return     the next element of this enumeration.
             * @exception  java.util.NoSuchElementException  if no more elements exist.
             */
            public Object nextElement() {
                return iterator.next();
            }
        }

    }
  }
